@@ -1,7 +1,8 @@
 # frozen_string_literal: true
-require_relative 'helper'
-require 'rubygems/commands/signin_command'
-require 'rubygems/installer'
+
+require_relative "helper"
+require "rubygems/commands/signin_command"
+require "rubygems/installer"
 
 class TestGemCommandsSigninCommand < Gem::TestCase
   def setup
@@ -34,7 +35,7 @@ class TestGemCommandsSigninCommand < Gem::TestCase
   end
 
   def test_execute_when_already_signed_in_with_same_host
-    host = 'http://some-gemcutter-compatible-host.org'
+    host = "http://some-gemcutter-compatible-host.org"
 
     util_capture(nil, host) { @cmd.execute }
     old_credentials = load_yaml_file Gem.configuration.credentials_path
@@ -46,10 +47,10 @@ class TestGemCommandsSigninCommand < Gem::TestCase
   end
 
   def test_execute_when_already_signed_in_with_different_host
-    api_key = 'a5fdbb6ba150cbb83aad2bb2fede64cf04045xxxx'
+    api_key = "a5fdbb6ba150cbb83aad2bb2fede64cf04045xxxx"
 
     util_capture(nil, nil, api_key) { @cmd.execute }
-    host = 'http://some-gemcutter-compatible-host.org'
+    host = "http://some-gemcutter-compatible-host.org"
 
     util_capture(nil, host, api_key) { @cmd.execute }
     credentials = load_yaml_file Gem.configuration.credentials_path
@@ -60,30 +61,55 @@ class TestGemCommandsSigninCommand < Gem::TestCase
   end
 
   def test_execute_with_host_supplied
-    host = 'http://some-gemcutter-compatible-host.org'
+    host = "http://some-gemcutter-compatible-host.org"
 
     sign_in_ui = util_capture(nil, host) { @cmd.execute }
     assert_match %r{Enter your #{host} credentials.}, sign_in_ui.output
     assert_match %r{Signed in.}, sign_in_ui.output
 
-    api_key     = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
+    api_key     = "a5fdbb6ba150cbb83aad2bb2fede64cf040453903"
     credentials = load_yaml_file Gem.configuration.credentials_path
     assert_equal api_key, credentials[host]
   end
 
+  def test_execute_with_host_permanent_redirect
+    host = "http://rubygems.example/"
+    ENV["RUBYGEMS_HOST"] = host
+    path = "/api/v1/api_key"
+    redirected_uri = "http://rubygems.example#{path}"
+    fetcher = Gem::FakeFetcher.new
+    fetcher.data["#{host}#{path}"] = HTTPResponseFactory.create(
+      body: "",
+      code: "308",
+      msg: "Permanent Redirect",
+      headers: { "location" => redirected_uri }
+    )
+    Gem::RemoteFetcher.fetcher = fetcher
+    ui = Gem::MockGemUi.new("you@example.com\nsecret\n\n\n\n\n\n\n\n\n")
+
+    assert_raise Gem::MockGemUi::TermError do
+      use_ui ui do
+        @cmd.execute
+      end
+    end
+
+    response = "The request has redirected permanently to #{redirected_uri}. Please check your defined push host URL."
+    assert_match response, ui.output
+  end
+
   def test_execute_with_valid_creds_set_for_default_host
     util_capture { @cmd.execute }
 
-    api_key     = 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
+    api_key     = "a5fdbb6ba150cbb83aad2bb2fede64cf040453903"
     credentials = load_yaml_file Gem.configuration.credentials_path
 
     assert_equal api_key, credentials[:rubygems_api_key]
   end
 
   def test_execute_with_key_name_and_scope
-    email     = 'you@example.com'
-    password  = 'secret'
-    api_key   = '1234'
+    email     = "you@example.com"
+    password  = "secret"
+    api_key   = "1234abcd"
     fetcher   = Gem::RemoteFetcher.fetcher
 
     key_name_ui = Gem::MockGemUi.new "#{email}\n#{password}\ntest-key\n\ny\n\n\n\n\n\n"
@@ -106,9 +132,9 @@ class TestGemCommandsSigninCommand < Gem::TestCase
   end
 
   def test_execute_with_key_name_scope_and_mfa_level_of_ui_only
-    email     = 'you@example.com'
-    password  = 'secret'
-    api_key   = '1234'
+    email     = "you@example.com"
+    password  = "secret"
+    api_key   = "1234abcd"
     fetcher   = Gem::RemoteFetcher.fetcher
     mfa_level = "ui_only"
 
@@ -133,9 +159,9 @@ class TestGemCommandsSigninCommand < Gem::TestCase
   end
 
   def test_execute_with_key_name_scope_and_mfa_level_of_gem_signin
-    email     = 'you@example.com'
-    password  = 'secret'
-    api_key   = '1234'
+    email     = "you@example.com"
+    password  = "secret"
+    api_key   = "1234abcd"
     fetcher   = Gem::RemoteFetcher.fetcher
     mfa_level = "ui_and_gem_signin"
 
@@ -159,20 +185,34 @@ class TestGemCommandsSigninCommand < Gem::TestCase
     assert_equal api_key, credentials[:rubygems_api_key]
   end
 
-  def test_execute_on_gemserver_without_profile_me_endpoint
-    host = 'http://some-gemcutter-compatible-host.org'
+  def test_execute_with_warnings
+    email     = "you@example.com"
+    password  = "secret"
+    api_key   = "1234abcd"
+    fetcher   = Gem::RemoteFetcher.fetcher
+    mfa_level = "disabled"
+    warning   = "/[WARNING/] For protection of your account and gems"
 
-    email     = 'you@example.com'
-    password  = 'secret'
-    api_key   = '1234'
+    key_name_ui = Gem::MockGemUi.new "#{email}\n#{password}\ntest-key\n\ny\n\n\n\n\n\ny"
+    util_capture(key_name_ui, nil, api_key, fetcher, mfa_level, warning) { @cmd.execute }
+
+    assert_match warning, key_name_ui.output
+  end
+
+  def test_execute_on_gemserver_without_profile_me_endpoint
+    host = "http://some-gemcutter-compatible-host.org"
+
+    email     = "you@example.com"
+    password  = "secret"
+    api_key   = "1234abcd"
     fetcher   = Gem::RemoteFetcher.fetcher
 
     key_name_ui = Gem::MockGemUi.new "#{email}\n#{password}\ntest-key\n\ny\n\n\n\n\n\ny"
 
     # Set the expected response for the Web-API supplied
-    ENV['RUBYGEMS_HOST']       = host
-    data_key                   = "#{ENV['RUBYGEMS_HOST']}/api/v1/api_key"
-    fetcher.data[data_key]     = [api_key, 200, 'OK']
+    ENV["RUBYGEMS_HOST"]       = host
+    data_key                   = "#{ENV["RUBYGEMS_HOST"]}/api/v1/api_key"
+    fetcher.data[data_key]     = HTTPResponseFactory.create(body: api_key, code: 200, msg: "OK")
 
     use_ui key_name_ui do
       @cmd.execute
@@ -193,18 +233,18 @@ class TestGemCommandsSigninCommand < Gem::TestCase
 
   # Utility method to capture IO/UI within the block passed
 
-  def util_capture(ui_stub = nil, host = nil, api_key = nil, fetcher = Gem::FakeFetcher.new, mfa_level = "disabled")
-    api_key        ||= 'a5fdbb6ba150cbb83aad2bb2fede64cf040453903'
-    response         = [api_key, 200, 'OK']
-    profile_response = [ "mfa: #{mfa_level}\n" , 200, 'OK']
-    email            = 'you@example.com'
-    password         = 'secret'
+  def util_capture(ui_stub = nil, host = nil, api_key = nil, fetcher = Gem::FakeFetcher.new, mfa_level = "disabled", warning = nil)
+    api_key        ||= "a5fdbb6ba150cbb83aad2bb2fede64cf040453903"
+    response         = HTTPResponseFactory.create(body: api_key, code: 200, msg: "OK")
+    profile_response = HTTPResponseFactory.create(body: "mfa: #{mfa_level}\nwarning: #{warning}", code: 200, msg: "OK")
+    email            = "you@example.com"
+    password         = "secret"
 
     # Set the expected response for the Web-API supplied
-    ENV['RUBYGEMS_HOST']       = host || Gem::DEFAULT_HOST
-    data_key                   = "#{ENV['RUBYGEMS_HOST']}/api/v1/api_key"
+    ENV["RUBYGEMS_HOST"]       = host || Gem::DEFAULT_HOST
+    data_key                   = "#{ENV["RUBYGEMS_HOST"]}/api/v1/api_key"
     fetcher.data[data_key]     = response
-    profile                    = "#{ENV['RUBYGEMS_HOST']}/api/v1/profile/me.yaml"
+    profile                    = "#{ENV["RUBYGEMS_HOST"]}/api/v1/profile/me.yaml"
     fetcher.data[profile]      = profile_response
     Gem::RemoteFetcher.fetcher = fetcher
 
