@@ -158,7 +158,7 @@ class TestGemCommandsUninstallCommand < Gem::InstallerTestCase
     @cmd.execute
 
     assert_equal false, File.exist?(formatted_executable)
-  rescue
+  rescue StandardError
     Gem::Installer.exec_format = nil
   end
 
@@ -229,6 +229,26 @@ class TestGemCommandsUninstallCommand < Gem::InstallerTestCase
     assert File.exist? File.join(@gemhome, "bin", "executable")
   end
 
+  def test_execute_with_multiple_version_specified_as_colon
+    initial_install
+
+    ui = Gem::MockGemUi.new "y\n"
+
+    util_make_gems
+
+    assert_equal 3, Gem::Specification.find_all_by_name("a").length
+
+    @cmd.options[:force] = true
+    @cmd.options[:args] = ["a:1", "a:2"]
+
+    use_ui ui do
+      @cmd.execute
+    end
+
+    assert_equal 1, Gem::Specification.find_all_by_name("a").length
+    assert_equal Gem::Version.new("3.a"), Gem::Specification.find_by_name("a").version
+  end
+
   def test_uninstall_selection
     ui = Gem::MockGemUi.new "1\n"
 
