@@ -205,7 +205,8 @@ class TestGemPackage < Gem::Package::TarTestCase
 
     tar.rewind
 
-    files, symlinks = [], []
+    files = []
+    symlinks = []
 
     Gem::Package::TarReader.new tar do |tar_io|
       tar_io.each_entry do |entry|
@@ -441,11 +442,11 @@ class TestGemPackage < Gem::Package::TarTestCase
     data_tgz = util_tar_gz {}
 
     gem = util_tar do |tar|
-      tar.add_file "data.tar.gz", 0644 do |io|
+      tar.add_file "data.tar.gz", 0o644 do |io|
         io.write data_tgz.string
       end
 
-      tar.add_file "metadata.gz", 0644 do |io|
+      tar.add_file "metadata.gz", 0o644 do |io|
         Zlib::GzipWriter.wrap io do |gzio|
           gzio.write @spec.to_yaml
         end
@@ -478,7 +479,7 @@ class TestGemPackage < Gem::Package::TarTestCase
     extracted = File.join @destination, "lib/code.rb"
     assert_path_exist extracted
 
-    mask = 0100666 & (~File.umask)
+    mask = 0o100666 & (~File.umask)
 
     assert_equal mask.to_s(8), File.stat(extracted).mode.to_s(8) unless
       win_platform?
@@ -488,11 +489,11 @@ class TestGemPackage < Gem::Package::TarTestCase
     data_tgz = util_tar_gz {}
 
     gem = util_tar do |tar|
-      tar.add_file "data.tar.gz", 0644 do |io|
+      tar.add_file "data.tar.gz", 0o644 do |io|
         io.write data_tgz.string
       end
 
-      tar.add_file "metadata.gz", 0644 do |io|
+      tar.add_file "metadata.gz", 0o644 do |io|
         Zlib::GzipWriter.wrap io do |gzio|
           gzio.write @spec.to_yaml
         end
@@ -522,14 +523,14 @@ class TestGemPackage < Gem::Package::TarTestCase
     filepath = File.join @destination, "README.rdoc"
     assert_path_exist filepath
 
-    assert_equal 0104444, File.stat(filepath).mode
+    assert_equal 0o104444, File.stat(filepath).mode
   end
 
   def test_extract_tar_gz_absolute
     package = Gem::Package.new @gem
 
     tgz_io = util_tar_gz do |tar|
-      tar.add_file "/absolute.rb", 0644 do |io|
+      tar.add_file "/absolute.rb", 0o644 do |io|
         io.write "hi"
       end
     end
@@ -547,12 +548,12 @@ class TestGemPackage < Gem::Package::TarTestCase
     package.verify
 
     tgz_io = util_tar_gz do |tar|
-      tar.add_file "relative.rb", 0644 do |io|
+      tar.add_file "relative.rb", 0o644 do |io|
         io.write "hi"
       end
 
-      tar.mkdir       "lib", 0755
-      tar.add_symlink "lib/foo.rb", "../relative.rb", 0644
+      tar.mkdir       "lib", 0o755
+      tar.add_symlink "lib/foo.rb", "../relative.rb", 0o644
     end
 
     begin
@@ -604,8 +605,8 @@ class TestGemPackage < Gem::Package::TarTestCase
     package.verify
 
     tgz_io = util_tar_gz do |tar|
-      tar.mkdir       "lib", 0755
-      tar.add_symlink "lib/foo.rb", "../broken.rb", 0644
+      tar.mkdir       "lib", 0o755
+      tar.add_symlink "lib/foo.rb", "../broken.rb", 0o644
     end
 
     ui = Gem::MockGemUi.new
@@ -624,9 +625,9 @@ class TestGemPackage < Gem::Package::TarTestCase
     package = Gem::Package.new @gem
 
     tgz_io = util_tar_gz do |tar|
-      tar.mkdir       "lib",               0755
-      tar.add_symlink "lib/link", "../..", 0644
-      tar.add_file    "lib/link/outside.txt", 0644 do |io|
+      tar.mkdir       "lib",               0o755
+      tar.add_symlink "lib/link", "../..", 0o644
+      tar.add_file    "lib/link/outside.txt", 0o644 do |io|
         io.write "hi"
       end
     end
@@ -692,11 +693,11 @@ class TestGemPackage < Gem::Package::TarTestCase
     package = Gem::Package.new @gem
 
     tgz_io = util_tar_gz do |tar|
-      tar.mkdir    "lib",        0755
-      tar.add_file "lib/foo.rb", 0644 do |io|
+      tar.mkdir    "lib",        0o755
+      tar.add_file "lib/foo.rb", 0o644 do |io|
         io.write "hi"
       end
-      tar.mkdir    "lib/foo", 0755
+      tar.mkdir    "lib/foo", 0o755
     end
 
     package.extract_tar_gz tgz_io, @destination
@@ -712,7 +713,7 @@ class TestGemPackage < Gem::Package::TarTestCase
     package = Gem::Package.new @gem
 
     tgz_io = util_tar_gz do |tar|
-      tar.add_file "./dot_slash.rb", 0644 do |io|
+      tar.add_file "./dot_slash.rb", 0o644 do |io|
         io.write "hi"
       end
     end
@@ -727,7 +728,7 @@ class TestGemPackage < Gem::Package::TarTestCase
     package = Gem::Package.new @gem
 
     tgz_io = util_tar_gz do |tar|
-      tar.add_file ".dot_file.rb", 0644 do |io|
+      tar.add_file ".dot_file.rb", 0o644 do |io|
         io.write "hi"
       end
     end
@@ -743,7 +744,7 @@ class TestGemPackage < Gem::Package::TarTestCase
       package = Gem::Package.new @gem
 
       tgz_io = util_tar_gz do |tar|
-        tar.add_file "foo/file.rb", 0644 do |io|
+        tar.add_file "foo/file.rb", 0o644 do |io|
           io.write "hi"
         end
       end
@@ -857,7 +858,7 @@ class TestGemPackage < Gem::Package::TarTestCase
 
   def test_verify_checksum_bad
     data_tgz = util_tar_gz do |tar|
-      tar.add_file "lib/code.rb", 0444 do |io|
+      tar.add_file "lib/code.rb", 0o444 do |io|
         io.write "# lib/code.rb"
       end
     end
@@ -867,11 +868,11 @@ class TestGemPackage < Gem::Package::TarTestCase
     gem = util_tar do |tar|
       metadata_gz = Gem::Util.gzip @spec.to_yaml
 
-      tar.add_file "metadata.gz", 0444 do |io|
+      tar.add_file "metadata.gz", 0o444 do |io|
         io.write metadata_gz
       end
 
-      tar.add_file "data.tar.gz", 0444 do |io|
+      tar.add_file "data.tar.gz", 0o444 do |io|
         io.write data_tgz
       end
 
@@ -881,7 +882,7 @@ class TestGemPackage < Gem::Package::TarTestCase
           "metadata.gz" => "bogus",
         },
       }
-      tar.add_file "checksums.yaml.gz", 0444 do |io|
+      tar.add_file "checksums.yaml.gz", 0o444 do |io|
         Zlib::GzipWriter.wrap io do |gz_io|
           gz_io.write Psych.dump bogus_checksums
         end
@@ -904,7 +905,7 @@ class TestGemPackage < Gem::Package::TarTestCase
 
   def test_verify_checksum_missing
     data_tgz = util_tar_gz do |tar|
-      tar.add_file "lib/code.rb", 0444 do |io|
+      tar.add_file "lib/code.rb", 0o444 do |io|
         io.write "# lib/code.rb"
       end
     end
@@ -914,7 +915,7 @@ class TestGemPackage < Gem::Package::TarTestCase
     gem = util_tar do |tar|
       metadata_gz = Gem::Util.gzip @spec.to_yaml
 
-      tar.add_file "metadata.gz", 0444 do |io|
+      tar.add_file "metadata.gz", 0o444 do |io|
         io.write metadata_gz
       end
 
@@ -927,13 +928,13 @@ class TestGemPackage < Gem::Package::TarTestCase
         },
       }
 
-      tar.add_file "checksums.yaml.gz", 0444 do |io|
+      tar.add_file "checksums.yaml.gz", 0o444 do |io|
         Zlib::GzipWriter.wrap io do |gz_io|
           gz_io.write Psych.dump checksums
         end
       end
 
-      tar.add_file "data.tar.gz", 0444 do |io|
+      tar.add_file "data.tar.gz", 0o444 do |io|
         io.write data_tgz
       end
     end
@@ -952,7 +953,7 @@ class TestGemPackage < Gem::Package::TarTestCase
     tf = Tempfile.open "corrupt" do |io|
       data = Gem::Util.gzip "a" * 10
       io.write \
-        tar_file_header("metadata.gz", "\000x", 0644, data.length, Time.now)
+        tar_file_header("metadata.gz", "\000x", 0o644, data.length, Time.now)
       io.write data
       io.rewind
 
@@ -1093,8 +1094,8 @@ class TestGemPackage < Gem::Package::TarTestCase
         build.add_metadata gem
         build.add_contents gem
 
-        gem.add_file_simple "a.sig", 0444, 0
-        gem.add_file_simple "a.sig", 0444, 0
+        gem.add_file_simple "a.sig", 0o444, 0
+        gem.add_file_simple "a.sig", 0o444, 0
       end
     end
 
@@ -1170,7 +1171,7 @@ class TestGemPackage < Gem::Package::TarTestCase
           def key; "key"; end
           def sign(*); "fake_sig"; end
         end
-        gem.add_file_signed "data2.tar.gz", 0444, fake_signer.new do |io|
+        gem.add_file_signed "data2.tar.gz", 0o444, fake_signer.new do |io|
           io.write bogus_data
         end
 
